"""Tool for managing archives of syslog logfiles.

Copyright (c) 2009, Florian Wagner <florian@wagner-flo.net>.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""

import os
import os.path as path
import bz2
import datetime
import syslog

import slarch.archive
import slarch.date


def append(filename, archive_pattern, date_pattern, find=None):
    syslog.syslog(
        'appending %s to archives %s' % (filename, archive_pattern))

    sigmask = slarch.date.SignificantMask(pattern=archive_pattern)
    archivefiles = slarch.date.SignificantDict(sigmask)

    def get_archivefile(date):
        date = datetime.datetime.strptime(date, date_pattern)
        archivefile = archivefiles.get(date, None)

        if archivefile is None:
            archivefile = slarch.archive.Archive.from_pattern(
                archive_pattern, date)
            archivefiles[date] = archivefile

        return archivefile

    datelen = slarch.date.pattern_len(date_pattern)
    archivefile = None
    lastdate = None
    
    if find is None:
        slice = lambda line: line[:datelen]
    else:
        slice = lambda line: line[line.find(find) + 1:][:datelen]
        
    for line in file(filename):
        try:
            date = slice(line)

            if date != lastdate:
                archivefile = get_archivefile(date)
                lastdate = date

        except:
            pass

        else:
            archivefile.append(line)


def compress(pattern, purge_delta, compress_delta):
    syslog.syslog(
        'starting compress and purge on %s' % pattern)

    now = datetime.datetime.now()

    for archive in slarch.archive.Archive.from_glob(pattern):
        date = datetime.datetime.strptime(archive.name, pattern)
        delta = (now.year * 12 + now.month) - (date.year * 12 + date.month)

        if delta > purge_delta:
            syslog.syslog('purging file %s' % archive.filename)
            archive.unlink()
            continue

        if delta > compress_delta:
            if archive.compress():
                syslog.syslog('compressed file %s' % archive.filename)
            continue
