"""Tool for managing archives of syslog logfiles.

Copyright (c) 2009, Florian Wagner <florian@wagner-flo.net>.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""

import os
import os.path as path
import bz2
import StringIO

import slarch.date

class PlainArchiveFile(file):
    suffix = ''
    is_compressed = False

class CompressedArchiveFile(StringIO.StringIO):
    is_compressed = True

    def __init__(self, name, mode='r'):
        self.name = name
        self.mode = mode
        self._dirty = False

        if 'r' in self.mode or 'a' in self.mode:
            StringIO.StringIO.__init__(self, self.read_data())
            
            if 'a' in self.mode:
                self.seek(0, os.SEEK_END)

        elif 'w' in self.mode:
            StringIO.StringIO.__init__(self)
            self._dirty = True

        else:
            raise ValueError("no 'r', 'w' or 'a' in mode string '%s'"
                             % self.mode)

    def write(self, s):
        StringIO.StringIO.write(self, s)
        self._dirty = True

    def writelines(self, iterable):
        StringIO.StringIO.writelines(self, iterable)
        self._dirty = True

    def truncate(self, size=None):
        StringIO.StringIO.truncate(self, size)
        self._dirty = True

    def close(self):
        if self._dirty:
            self.write_data()
        StringIO.StringIO.close(self)

    def __del__(self):
        self.close()

class BzipArchiveFile(CompressedArchiveFile):
    suffix = '.bz2'
    
    def read_data(self):
        if not path.exists(self.name):
            return ''

        return bz2.BZ2File(self.name).read()
    
    def write_data(self):
        bz2.BZ2File(self.name, 'w').write(self.getvalue())
   

class Archive(object):
    _fobj_classes = [
        BzipArchiveFile,
        PlainArchiveFile,
    ]
    _fobj_default_compressed = BzipArchiveFile

    def __init__(self, name):
        self.fcls = None
        self.filename = name
        
        # first: check if an archive of the given name exists
        for cls in self._fobj_classes:
            # append suffix of current class if it is not already there
            if not self.filename.endswith(cls.suffix):
                filename = self.filename + cls.suffix
            else:
                filename = self.filename
                
            # see if a file of this name exists and if yes use it and
            # the current class
            if path.exists(filename):
                self.fcls = cls
                self.filename = filename
                break

        # second (no file found): check the given name for one of the
        # suffixes and on match use that class
        if self.fcls is None:
            for cls in self._fobj_classes:
                if self.filename.endswith(cls.suffix):
                    self.fcls = cls
                    break

        self.fobj = self.fcls(self.filename, mode='a')
        
    @classmethod
    def from_pattern(cls, pattern, date):
        return cls(date.strftime(pattern))

    @classmethod
    def from_glob(cls, pattern):
        for fcls in cls._fobj_classes:
            for filename in slarch.date.pattern_glob(pattern + fcls.suffix):
                yield cls(filename)

    @property
    def name(self):
        if self.fcls.suffix == '':
            return self.filename
        
        return self.filename[:-len(self.fcls.suffix)]
        
    def append(self, s):
        self.fobj.write(s)

    def appendlines(self, iterable):
        self.fobj.writelines(iterable)

    def compress(self):
        if self.fobj.is_compressed:
            return False
        
        # create compressed archive file object
        cmpr = self._fobj_default_compressed(
            self.filename + self._fobj_default_compressed.suffix, mode='a')

        # append date from current archive file object
        self.fobj.close()
        uncmpr = self.fcls(self.filename, mode='r')
        cmpr.writelines(uncmpr.readlines())
        
        # close current archive file and delete it
        uncmpr.close()
        os.unlink(self.filename)

        # make compressed archive file object the current one
        self.fobj = cmpr
        self.filename = cmpr.name

        return True

    def unlink(self):
        self.fobj.close()
        os.unlink(self.filename)
