"""Tool for managing archives of syslog logfiles.

Copyright (c) 2009, Florian Wagner <florian@wagner-flo.net>.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""

import re
import os
import os.path as path
import calendar

class SignificantMask(object):
    def __init__(self, pattern=None, slist=None):
        if pattern is None and slist is None:
            raise Exception()

        if pattern is not None and slist is not None:
            raise Exception()

        if pattern is not None:
            slist = significant_list(pattern)

        self._slist = sorted(slist)

    def compare(self, dtime1, dtime2):
        for attr in self._slist:
            if getattr(dtime1, attr) != getattr(dtime2, attr):
                return False

        return True

    def hashid(self, dtime):
        return tuple(getattr(dtime, attr) for attr in self._slist)


class SignificantDict(dict):
    def __init__(self, sigmask, *args, **kwargs):
        self._sigmask = sigmask
        dict.__init__(self, *args, **kwargs)

    def __getitem__(self, key):
        return dict.__getitem__(self, self._sigmask.hashid(key))

    def __setitem__(self, key, value):
        dict.__setitem__(self, self._sigmask.hashid(key), value)

    def __delitem(self, key):
        dict.__delitem__(self, self._sigmask.hashid(key))

    def __contains__(self, key):
        return dict.__contains__(self, self._sigmask.hashid(key))

    def get(self, key, default=None):
        return dict.get(self, self._sigmask.hashid(key), default)
        
_significant_map = {
    ('%Y', '%y'): 'year',
    ('%m', '%b'): 'month',
    ('%d', '%e'): 'day',
    ('%H',): 'hour',
    ('%M',): 'minute',
    ('%S',): 'second',
}

def significant_list(pattern):
    pattern = pattern.replace('%%', '')
    result = []
    
    for codelist,attr in _significant_map.iteritems():
        if attr in result:
            continue

        for code in codelist:
            if code not in pattern:
                continue

            result.append(attr)
            break

    return result

_glob_map = {
    r'\%\%': r'\%',
    r'\%Y': r'[12]\d{3}',
    r'\%y': r'\d{2}',
    r'\%m': r'(0[123456789]|1[012])',
    r'\%b': r'(%s)' % '|'.join(re.escape(i) for i in calendar.month_abbr[1:]),
    r'\%d': r'(0[123456789]|[12]\d|3[01])',
    r'\%e': r'(0[123456789]|[12]\d|3[01])',
    r'\%H': r'([01]\d|2[0123])',
    r'\%M': r'([01234]\d|5[0123456789])',
    r'\%S': r'[0123456]\d',
}

def _toregex(pattern):
    nreplminus = [0]
    
    def repl(match):
        match = match.group(0)
        if match == r'\%\%':
            nreplminus[0] = nreplminus[0] + 1

        return _glob_map[match]
    
    pattern,nrepl = re.subn(r'\\\%\\\%|\\\%[a-zA-Z]', repl,
                            re.escape(pattern))
    return (('^%s$' % pattern), nrepl - nreplminus[0])

def pattern_glob(pattern):
    head = pattern
    rest = []

    while True:
        head,tail = path.split(head)
        rest.insert(0, re.compile(_toregex(tail)[0]))

        nhead,nrepl = _toregex(head)
        if nrepl == 0:
            head = head.replace('%%', '%')
            break

    top = head != '' and head or '.'
    dpat = rest[:-1]
    fpat = rest[-1]

    for root,dirs,files in os.walk(top):
        if dpat != []:
            pattern = dpat.pop(0)
            newdirs = []
            
            for dir in dirs[:]:
                if pattern.match(dir):
                    newdirs.append(dir)

            dirs[:] = newdirs

        else:
            # slice assignment: make the list referenced by the dirs
            # variable empty
            dirs[:] = []

            # match filenames to pattern
            for filename in files:
                if fpat.match(filename) is None:
                    continue

                if head == '':
                    root = root[len(os.sep) + 1:]
                    
                yield path.join(root, filename)

_len_map = [
    ('%%', 1),
    ('%Y', 4),
    ('%y', 2),
    ('%m', 2),
    ('%b', len(calendar.month_abbr[1])),
    ('%d', 2),
    ('%e', 2),
    ('%H', 2),
    ('%M', 2),
    ('%S', 2),
]

def pattern_len(pattern):
    for search,repl in _len_map:
        pattern = pattern.replace(search, '-' * repl)

    return len(pattern)
